#ifndef INCLUDE_CODEFILTER_UTIL_H
#define INCLUDE_CODEFILTER_UTIL_H

#include "clang/AST/ASTContext.h"
#include "clang/AST/Decl.h"
#include "clang/Basic/SourceManager.h"
#include "llvm/ADT/SmallPtrSet.h"

#include "clang/AST/Expr.h"

#include <string>
#include <iostream>

namespace clang {
    class CallExpr;

}  // namespace clang

template<typename T>
using PtrSet = llvm::SmallPtrSet<T, 16>;

std::string get_decl_name(const clang::NamedDecl *d, clang::ASTContext &ctx);

std::string get_decl_name(const clang::NamedDecl *d);

std::string stmt2str(const clang::Stmt *st, clang::SourceManager &sm);

template<typename T>
// Ughh the ast nodes do not share a base class
std::string dump_to_string(const T *stmt) {
    std::string ret;
    llvm::raw_string_ostream ss(ret);
    stmt->dump(ss);
    ss.flush();
    return ret;
}

static std::vector<std::string> getKnownHdrExts() {
    return {"h", "H", "hpp", "HPP", "hxx", "HXX"};
}

template<typename C, typename T>
inline bool contains(const C &c, const T &val) {
    return std::find(c.begin(), c.end(), val) != c.end();
}

/*
template <typename T>
std::string col2str(const T &col, const std::string &prefix = "", const std::string &split = ",",
                    const std::string &suffix = ""){
    std::string ret=prefix;
    for (auto begin = col.begin(), end = std::prev(col.end()); begin != end; begin++) {
        ret += *begin;
        ret += split;
    }
    ret += col.back();;
    ret += suffix;
    return ret;
}
*/
template<typename T>
std::string col2str(const T &col, const std::string &prefix = "", const std::string &split = ",",
                    const std::string &suffix = "", std::string (*transformer)(
        typename std::remove_reference<decltype(*std::declval<T>().begin())>::type) = [](
        typename std::remove_reference<decltype(*std::declval<T>().begin())>::type t) -> std::string { return t; }) {
    std::string ret = prefix;

    for (auto begin = col.begin(), end = std::prev(col.end()); begin != end; begin++) {
        ret += transformer(*begin);
        ret += split;
    }
    ret += transformer(col.back());

    ret += suffix;
    return ret;
}

void triml(std::string &str);

void trimr(std::string &str);

std::string trim(std::string str);

/**
 * Returns the source location where the file containing loc is included in the main file
 * @param loc The location to find the include for
 * @param sm  Source manager
 * @return Soruce location. Can be invalid if not found
 */
clang::SourceRange get_include_in_main_file(const clang::SourceRange &loc, const clang::SourceManager &sm);

/**
 * Returns the diretly underlying type if t is a pointer or array
 * @param t
 * @return
 */
clang::QualType get_underlying_type(clang::QualType t);

/**
 * Returns a type string representing
 * @param t
 * @return
 */
std::string get_elem_ptr_string(clang::QualType t);

bool is_potential_mod(const clang::QualType &t);

#endif
