//
// Created by tim on 21.12.23.
//
#include "GlobalsMarker.h"
#include "iostream"
#include "clang/Basic/SourceManager.h"


bool GlobalsMarker::VisitDeclRefExpr(clang::DeclRefExpr *decl_ref){
    assert(decl_ref);
    auto *decl = decl_ref->getDecl();

    if(!decl_ref->getDecl()->getASTContext().getSourceManager().isInMainFile(decl_ref->getDecl()->getSourceRange().getBegin())){
        //std::cout<<"We reference a declaration that is part of a header, we don't need to extract it\n";
        return true;
    }

    if (auto *var_decl = clang::dyn_cast<clang::VarDecl>(decl); var_decl && var_decl->hasGlobalStorage() && !var_decl->isStaticLocal()) {
        outlineStore.insert(var_decl);
        //outline_store.set_var_to_outline(var_decl);
        this->TraverseVarDecl(var_decl);
        //this->TraverseVarDecl(var_decl);
    }
    return true;
}

bool GlobalsMarker::VisitType(clang::Type *t) {

    if (auto *tag = t->getAsTagDecl()) {
        if(!tag->getASTContext().getSourceManager().isInMainFile(tag->getLocation())){
            //SPDLOG_TRACE("Tag is not user defined\n");
            return true;
        }
        //if (!outline_store.contains_type(tag)) {
        if (!outlineStore.count(tag)) {
            outlineStore.insert(tag);
            this->TraverseDecl(tag);
        }
    }

    if (auto *typedf = clang::dyn_cast<clang::TypedefType>(t)) {
        auto typedf_decl = typedf->getDecl();
        assert(typedf_decl);
        if(!typedf_decl->getASTContext().getSourceManager().isInMainFile(typedf_decl->getLocation())){
            //SPDLOG_TRACE("Typedf_decl is not user defined\n");
            return true;
        }
        if (!outlineStore.count(typedf_decl)) {
            //if (!outline_store.contains_type(typedf_decl)) {
            outlineStore.insert(typedf_decl);
            this->TraverseDecl(typedf_decl);
        }
    }
    return true;
}

std::unordered_set<clang::Decl *> GlobalsMarker::get_nodes_to_outline(clang::FunctionDecl *func_decl) {
    assert(func_decl);
    if(!func_decl->hasBody()){
        std::cout<<"The function: "<<func_decl->getNameAsString()<<" has no body (predecl)\n";
        return {};
    }
    if(!func_decl->Decl::hasBody()){
        assert(false && "This check should be superfluous");
    }
    //outline_store.clear();
    outlineStore.clear();

    TraverseFunctionDecl(func_decl);
    return outlineStore;
}