//
// Created by tim on 03.12.24.
//

#ifndef CTUAPEX_GEOMETRY_H
#define CTUAPEX_GEOMETRY_H
#include <iostream>
namespace Geometry {

    // Struct
    struct Point {
        int x, y;
        Point(int x, int y);
    };

    // Base class for Virtual Inheritance
    class Shape {
    public:
        virtual void draw() const;
        virtual double area() const = 0;
        virtual ~Shape() = default;
    };

    // Virtual Inheritance
    class TwoDimensionalShape : public virtual Shape {
    public:
        virtual double perimeter() const = 0;
    };

    // Derived class from TwoDimensionalShape
    class Circle : public TwoDimensionalShape {
    private:
        Point center;
        double radius;
    public:
        Circle(int x, int y, double r);

        // Overriding virtual functions
        void draw() const override;

        double area() const override;
        double perimeter() const override;

        // Operator Overloading (for output)
        friend std::ostream& operator<<(std::ostream& os, const Circle& c){
            os << "Circle at (" << c.center.x << ", " << c.center.y << "), radius " << c.radius;
            return os;
        }

        // Constructor Overloading
        Circle(double r);

        // Function Overloading
        void setRadius(double r);
        void setRadius(int r) ;
    };

    // Template Class
    template <typename T>
    class Box {
    private:
        T value;
    public:
        Box(T value) : value(value) {}
        T getValue() const { return value; }
    };

    // Template Function
    template <typename T>
    T add(T a, T b) {
        return a + b;
    }

    // Static and Non-static Variables
    class Counter {
    public:
        static int staticCount;
        int nonStaticCount;

        Counter();

        void increment();

        static void reset();
    };


    // Function Pointer
    typedef void (*DrawFunc)(const Shape&);

    void printShape(const Shape& shape);
}

#endif //CTUAPEX_GEOMETRY_H
