//
// Created by tim on 16.12.24.
//
#include "uses.h"
#include <iostream>
#include "geometry.h"
#include "macros.h"

// Global variable
int globalVar = 100;

//Define macro in main file
//We would need to extract this, but we can rely on the header files to supply other macro definitions
#define MY_ASSERT(BOOL_EXPRESSION) \
    do { \
        _Pragma("warning(suppress: 4127)") /* C4127 conditional expression is constant */  \
        if (!(BOOL_EXPRESSION)) {   \
            printf("MY_ASSERT FAILED: \"" #BOOL_EXPRESSION "\" on %s(%d)", __FILE__, __LINE__); \
            exit(-1); \
        } \
    } while (0)

// Lambda Function
auto multiply = [](int a, int b) { return a * b; };


void useGlobal(){

    // Global variable usage
    std::cout << "Global Variable: " << globalVar << std::endl;
}

void useShapes(){

    // Creating a Circle object
    Geometry::Circle circle1(10);
    circle1.draw();
    std::cout << "Circle Area: " << circle1.area() << ", Perimeter: " << circle1.perimeter() << std::endl;

    // Using Operator Overloading (for Circle)
    std::cout << "Circle: " << circle1 << std::endl;

    // Template Class usage
    Geometry::Box<int> intBox(42);
    std::cout << "Box value: " << intBox.getValue() << std::endl;
}

void useGeometry(){
    useShapes();
    // Template Function usage
    std::cout << "Add(3, 5): " << Geometry::add(3, 5) << std::endl;
}

#ifdef Statics
void useStatics(){
    // Static and Non-static variables
    Geometry::Counter counter;
    counter.increment();
    std::cout << "Static count: " << Geometry::Counter::staticCount << ", Non-static count: " << counter.nonStaticCount << std::endl;

}
#endif

void usePtrCall(){
    // Using Function Pointer
    Geometry::Circle circle1(10);
    Geometry::DrawFunc drawFunc = Geometry::printShape;
    drawFunc(circle1); // Calling the printShape function via pointer

    MY_ASSERT(true);
}

void useLambdaFunction(){

    // Lambda Function
    std::cout << "Lambda Multiply: " << multiply(4, 5) << std::endl;
}